@@ -13,7 +13,8 @@ WC_NOTIFY_SERVICE=/etc/systemd/system/wc-notify.service
 
 cat > $WC_NOTIFY_BIN <<EOF
 #!/bin/bash -v
-until curl -sf "http://127.0.0.1:8080/healthz"; do
+until  [ "ok" = "$(curl --silent http://127.0.0.1:8080/healthz)" ]
+do
     echo "Waiting for Kubernetes API..."
     sleep 5
 done
@@ -23,8 +24,8 @@ EOF
 cat > $WC_NOTIFY_SERVICE <<EOF
 [Unit]
 Description=Notify Heat
-After=docker.service etcd.service
-Requires=docker.service etcd.service
+After=docker.service
+Requires=docker.service
 [Service]
 Type=oneshot
 ExecStart=$WC_NOTIFY_BIN
